/*
 *
 * AMX Mod X Module
 * Basic Socket Functions
 * 
 * Codebase from Ivan, -g-s-ivan@web.de (AMX 0.9.3)
 * Modification by Olaf Reusch, kenterfie@hlsw.de (AMXX 0.16, AMX 0.96)
 * Modification by David Anderson, dvander@tcwonline.org (AMXx 0.20)
 * Modification by hackziner,  hackziner@hotmail.com
 * Modification by Destro, ema_destro_cs@hotmail.com (Only-Arg)
 *
 * Bugs/Fixes
 *
 * v0.1
 * - code structure renewed  (HLSW Dev Team)
 * v0.2
 * - added socket_send2 to send data containing null bytes (FALUCO)(AMXX v1.65) (HLSW Dev Team)
 * v0.3
 * - accept/listen/block (hackziner)
 * v0.4
 * - recvfrom/sendto (Destro)
 */

#if defined _socket3_included
	#endinput
#endif
#define _socket3_included

#if AMXX_VERSION_NUM >= 175
 #pragma reqlib sockets3
 #if !defined AMXMODX_NOAUTOLOAD
  #pragma loadlib sockets3
 #endif
#else
 #pragma library socket3
#endif


// Use SOCKET_TCP for TCP Socket connections
#define SOCKET_TCP 1

// Use SOCKET_UDP for UDP Socket connections
#define SOCKET_UDP 2

/* Opens a new connection to hostname:port via protocol (either SOCKET_TCP or SOCKET_UDP),
 * returns a socket (positive) or negative or zero on error.
 * States of error:
 * 0 - no error
 * 1 - error while creating socket
 * 2 - couldn't resolve hostname
 * 3 - couldn't connect to given hostname:port 
*/
native socket_open(const _hostname[], _port, _protocol = SOCKET_TCP, &_error);


/* Opens a new listenning connection bind to hostname:port via protocol (either SOCKET_TCP or SOCKET_UDP),
 * returns a socket (positive) or negative or zero on error.
 * States of error:
 * 0 - no error
 * 1 - error while creating socket
 * 2 - couldn't resolve hostname
 * 3 - couldn't connect to given hostname:port 
*/
native socket_listen(const _hostname[], _port, _protocol = SOCKET_TCP, &_error);


/* Closes a Socket */
native socket_close(_socket);


/* Accept a connection on a listenning socket 
 * returns a socket (positive) or negative or zero on error. */
native socket_accept(_socket);


/* Set a socket in unblocking mode */
native socket_unblock(_socket);


/* Recieves Data to string with the given length */
native socket_recv(_socket, _data[], _length);


/* Sends data to the Socket */
native socket_send(_socket, const _data[], _length);


/* Same as socket_send but Data can contain null bytes */
native socket_send2(_socket, const _data[], _length);


/* This function will return true if the state (buffer content) have changed within the last recieve or
* the timeout, where timeout is a value in Seconds, (1 sec =1000000 sec). 
* Use to check if new data is in your socket. */
native socket_change(_socket, _timeout=100000);


/* Recieves Data to string with the given length */
native socket_recvfrom(_socket, _data[], _len, &_port, _ip[], _iplen);


/* Sends data to specific server */
native socket_sendto(_socket, _data[], _ip[], _port);
/* AMXX-Studio Notes - DO NOT MODIFY BELOW HERE
*{\\ rtf1\\ ansi\\ deff0{\\ fonttbl{\\ f0\\ fnil Tahoma;}}\n\\ viewkind4\\ uc1\\ pard\\ lang11274\\ f0\\ fs16 \n\\ par }
*/
